/*
   RecLocks.prg
   A sample dBL program from 
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2012. 
   Permission granted for use as a learning tool for dBASE Plus.

   Sample from: Chapter 10 
   Example of the using session objects and testing for record 
   locks ... this is how I was able to obtain the screen shot
   shown in the book for this chapter.

   This code is more complex than one might expect, simply
   because I am using session objects, which then require that
   the User BDE Alias be added to them, which backing up to the 
   top of the program is why the call to qBDEMgr.cc is necessary.

   This could be simplified dramatically by opening two 
   sessions of dBASE and in the Command Window of each type:

      open database DBASESAMPLES
      set database to DBASESAMPLES
      use fish
      go top
      lock()

   Issue the same commands in the second session of dBASE's
   Command Window, and you will see the error. 

   However, that does not demonstrate the use of the session 
   objects, the User BDE Aliases, and it does not show other 
   interesting concepts you can learn by reading the code below. 
   <grin> 
*/

// get path to the Samples folder, this will be used
// in a temporary database alias as shown below:

try // in case of error with database

   set procedure to :NonVisual:qBDEMgr.cc
   oBDE = new qBDEManager()
   oDbDesc = oBDE.GetAliasDesc( "DBASESAMPLES" )
   cPath = oDBDesc.szPhyName()  // Path to the database

catch( exception e )
   msgbox( "There may be a problem with your "+chr(13)+;
           "installation of dBASE Plus. The "+chr(13)+;
           "database 'DBASESAMPLES' does not "+chr(13)+;
           "exist. Check your PLUS.INI file.",;
           "Database Does Not Exist", 16)
   cancel

endtry

// cleanup this part of the code (get rid of
// object references not needed):
close procedure :NonVisual:qBDEMgr.cc
oDBDesc := null
oBDE    := null
release oDBDesc
release oBDE

// set up first session, User BDE Alias, database, and
// query:
s1 = new Session()
s1.addAlias( "DBASESAMPLES", "DBASE", "Path: "+cPath )
d1 = new database()
d1.databaseName := "DBASESAMPLES"
d1.session      := s1
d1.active       := true
q1 = new query()
q1.session      := s1
q1.database     := d1
q1.sql          := "select * from fish"
q1.active       := true
q1.rowset.first()

// set up second session, User BDE Alias, database, and
// query:
s2 = new Session()
s2.addAlias( "DBASESAMPLES", "DBASE", "Path: "+cPath )
d2 = new database()
d2.databaseName := "DBASESAMPLES"
d2.session      := s2
d2.active       := true
q2 = new query()
q2.session      := s2
q2.database     := d2
q2.sql          := "select * from fish"
q2.active       := true
q2.rowset.first()

// now we have the objects, edit in first session:
q1.rowset.beginEdit()
// attempt to edit in second session:
q2.rowset.beginEdit()

// message should now be on screen about attempting
// to lock the row, etc. User clicks the "Cancel" button
// and the dialog closes, the program continues ...

// in a real-world environment, when the user who has Session 1
// active releases the lock on the record, then the lock can be
// obtained by the user with Session 2 ...

q2.active := false
d2.active := false

q1.active := false
d1.active := false

release object s1
release object s2
release object d1
release object d2
release object q1
release object q2

// end of program ...
