/*
   MakeChapter8Tables.prg
   A sample dBL program from 
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2005-2012. 
   Permission granted for use as a learning tool for dBASE Plus.

   Sample from: Chapter 8 
   Create parent/child tables used in examples in this chapter.
   This should be run by the reader before trying the other
   code. While they could try building this program, the potential
   for errors is huge, so even though there is a complete program
   listing in the book, it is better to just run this.
*/

// close any open databases and tables
close database
// if tables by these names exist, delete them:
if file( "ParentTest.dbf" )
   drop table ParentTest
endif
if file( "ChildTest.dbf" )
   drop table ChildTest
endif
if file( "TestMorph.dbf" )
   drop table TestMorph
endif
if file( "AddressBook.dbf" )
   drop table AddressBook
endif
if file( "StateProv.dbf" )
   drop table StateProv
endif


clear
? "Creating sample tables"

// create the parent table, and add some rows:
create table ParentTest (;
             LinkField integer,;
             ParentData char(10) )
use ParentTest
append blank
replace LinkField with 1, ParentData with "Parent 1"
append blank
replace LinkField with 2, ParentData with "Parent 2"
append blank
replace LinkField with 3, ParentData with "Parent 3"
append blank
replace LinkField with 4, ParentData with "Parent 4"
use // close table
create index LinkField on ParentTest (LinkField)

// create the child table, and add some rows
create table ChildTest (;
             LinkField integer,;
             ChildData char(20) )
create index LinkField on ChildTest (LinkField)
use ChildTest
append blank
// child data is not 100% "in sequence" intentionally
replace LinkField with 1, ChildData with "Parent 1, Child 1"
append blank
replace LinkField with 1, ChildData with "Parent 1, Child 3"
append blank
replace LinkField with 1, ChildData with "Parent 1, Child 2"
append blank
replace LinkField with 2, ChildData with "Parent 2, Child 1"
append blank
replace LinkField with 3, ChildData with "Parent 3, Child 2"
append blank
replace LinkField with 3, ChildData with "Parent 3, Child 1"
append blank
replace LinkField with 4, ChildData with "Parent 4, Child 1"
append blank
replace LinkField with 4, ChildData with "Parent 4, Child 5"
append blank
replace LinkField with 4, ChildData with "Parent 4, Child 3"
append blank
replace LinkField with 4, ChildData with "Parent 4, Child 2"
append blank
replace LinkField with 4, ChildData with "Parent 4, Child 4"
use

// Create TestMorph table
create table TestMorph ( MyLogical numeric(1) )

// Put some data in the table
use TestMorph
append blank
replace MyLogical with 1
append blank
replace MyLogical with 0
append blank
replace MyLogical with 1
use

// Create AddressBook table
create table AddressBook (;
             FirstName      char( 15 ),;
             LastName       char( 20 ),;
             Address1       char( 25 ),;
             Address2       char( 25 ),;
             City           char( 30 ),;
             StateProv      char(  3 ),;
             PostalCode     char( 10 ),;
             Country        char(  3 ),;
             Phone          char( 14 ),;
             Business       boolean,;
             Notes          blob( 10, 1 ),;
             Birthday       date,;
             FavoriteFlavor char( 25 );
                          )

// Create index:
local i 
i = new DBFIndex()
i.expression := "upper( LastName )+upper( FirstName )"
i.indexName  := "Name"
_app.databases[1].createIndex( "AddressBook", i )

// XDML is faster for some things:
use AddressBook
// first row:
append blank
replace FirstName      with "Fred"
replace LastName       with "Smith"
replace Address1       with "1234 First Street"
replace City           with "Testville"
replace StateProv      with "CA"
replace PostalCode     with "99999-9999"
replace Country        with "USA"
replace Phone          with "(999) 999-9999"
replace Business       with false
replace Notes          with "Fred is an old family friend, his wife's name is Judy. No children at this time."
replace Birthday       with {03/31/1963}
replace FavoriteFlavor with "Chocolate"
// Second row
append blank
replace FirstName      with "Joe"
replace LastName       with "Jones"
replace Address1       with "111 Main St."
replace Address2       with "Box 5678"
replace City           with "Somecity"
replace StateProv      with "OR"
replace PostalCode     with "88888-8888"
replace Country        with "USA"
replace Phone          with "(888) 888-9999"
replace Business       with true
replace Notes          with "Joe's business needs some help, and he contacted me ..."
replace Birthday       with {07/15/1960}
replace FavoriteFlavor with "Macadamia Nut"
// close the table
use

// Create StateProv table:
create table StateProv (;
             Code      char(3),;
             StateProv char( 30 ) )

i = new DBFIndex()
i.expression := "upper( StateProv )"
i.indexName  := "StateProv"
_app.databases[1].createIndex( "StateProv", i )

use StateProv
append blank
replace Code      with "CA"
replace StateProv with "California"
append blank
replace code      with "OR"
replace StateProv with "Oregon"
append blank
replace code      with "WA"
replace StateProv with "Washington"
append blank
replace code      with "AZ"
replace StateProv with "Arizona"
append blank
replace code      with "ID"
replace StateProv with "Idaho"
append blank
replace code      with "NM"
replace StateProv with "New Mexico"
use
// normally we would add all 50 states in the US, possibly
// the provinces in Canada, etc. We're keeping it simple!


? "Done"